/*
 * io_install.c
 *
 *  Created on: 08.06.2011
 *      Author: gen4ik
 */

#include "main.h"

char io_start_twi(void)
{
	static const gpio_map_t TWI_GPIO_MAP =
	{
			{AVR32_TWI_SDA_0_0_PIN, AVR32_TWI_SDA_0_0_FUNCTION},
			{AVR32_TWI_SCL_0_0_PIN, AVR32_TWI_SCL_0_0_FUNCTION}
			//{29, 0},
			//{30, 0}
	};

	twi_options_t opt;

	gpio_enable_module(TWI_GPIO_MAP, sizeof(TWI_GPIO_MAP) / sizeof(TWI_GPIO_MAP[0]));

	opt.pba_hz = 33000000;
	opt.speed = TWI_SPEED;
	opt.chip = TWI_CAM_ADDRESS;


	if (twi_master_init(&AVR32_TWI, &opt) == TWI_SUCCESS)
	{
		return 0;
	}
	else
	{
		return -1;
	}

}
char io_start_spi(void)
{

	static const gpio_map_t SPI0_GPI0_MAP = {
				  { AVR32_SPI0_SCK_0_PIN,  AVR32_SPI0_SCK_0_FUNCTION }, //SPI Clock
				  { AVR32_SPI0_MISO_0_PIN, AVR32_SPI0_MISO_0_FUNCTION }, //MISO
				  { AVR32_SPI0_MOSI_0_PIN, AVR32_SPI0_MOSI_0_FUNCTION }, //MOSI
				  { AVR32_SPI0_NPCS_0_PIN, AVR32_SPI0_NPCS_0_FUNCTION }
		  };

		  //add the spi options driver structure for the SPARE SPI
		  spi_options_t spi0Options = { 	.reg= 0,
											.baudrate=33000000,
											.bits=8,
											.spck_delay=0,
											.trans_delay=0,
											.stay_act=1,//1
											.spi_mode=0,
											//.fdiv=1,
											.modfdis=1 };



	gpio_enable_module(SPI0_GPI0_MAP, sizeof(SPI0_GPI0_MAP) / sizeof(SPI0_GPI0_MAP[0]));

	if( SPI_OK != spi_initMaster( &AVR32_SPI0, &spi0Options ))
	      return 1;

	if( SPI_OK != spi_selectionMode( &AVR32_SPI0, 0, 0, 0 ) )
	      return 2;

	spi_enable(&AVR32_SPI0);

	//Hier neues CHIP einfügen !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	if( SPI_OK != spi_setupChipReg( &AVR32_SPI0, &spi0Options, 66000000 ) )
	      return 3;

	//spi_selectChip( &AVR32_SPI0, 0 );
	spi_unselectChip( &AVR32_SPI0, 0 );

	return 0;

}

void io_start_clock(void)
{
	volatile avr32_pm_t* pm = &AVR32_PM;
	  /* \note All calculations here suppose that the Osc0 frequency is 12MHz. */

	  pm_switch_to_osc0(pm, FOSC0, OSC0_STARTUP);  // Switch main clock to Osc0.

	  /* Setup PLL0 on Osc0, mul=7 ,no divisor, lockcount=16, ie. 12Mhzx8 = 96MHz output */
	  pm_pll_setup(pm,
	               0,   // use PLL0
	              13,   // MUL=7 in the formula//13,14 10->66Mhz
	               1,   // DIV=1 in the formula
	               0,   // Sel Osc0/PLL0 or Osc1/PLL1
	               16); // lockcount in main clock for the PLL wait lock

	  /*
	   This function will set a PLL option.
	   *pm Base address of the Power Manager (i.e. &AVR32_PM)
	   pll PLL number 0
	   pll_freq Set to 1 for VCO frequency range 80-180MHz, set to 0 for VCO frequency range 160-240Mhz.
	   pll_div2 Divide the PLL output frequency by 2 (this settings does not change the FVCO value)
	   pll_wbwdisable 1 Disable the Wide-Bandith Mode (Wide-Bandwith mode allow a faster startup time and out-of-lock time). 0 to enable the Wide-Bandith Mode.
	  */
	  /* PLL output VCO frequency is 96MHz. We divide it by 2 with the pll_div2=1. This enable to get later main clock to 48MHz */
	  pm_pll_set_option(pm, 0, 1, 1, 0);

	  /* Enable PLL0 */
	  /*
	    void pm_pll_enable(volatile avr32_pm_t* pm,
	                  unsigned int pll) {
	  */
	  pm_pll_enable(pm,0);

	  /* Wait for PLL0 locked */
	  pm_wait_for_pll0_locked(pm) ;

	  /* Divide PBA clock by 2 from main clock (PBA clock = 48MHz/2 = 24MHz).
	     Pheripheral Bus A clock divisor enable = 1
	     Pheripheral Bus A select = 0
	     Pheripheral Bus B clock divisor enable = 0
	     Pheripheral Bus B select = 0
	     High Speed Bus clock divisor enable = 0
	     High Speed Bus select = 0
	  */
	  pm_cksel(pm, 0, 0, 0, 0, 0, 0);

	  // Set one wait-state (WS) for flash controller. 0 WS access is up to 30MHz for HSB/CPU clock.
	  // As we want to have 48MHz on HSB/CPU clock, we need to set 1 WS on flash controller.
	  flashc_set_wait_state(1);

	  pm_switch_to_clock(pm, AVR32_PM_MCSEL_PLL0); /* Switch main clock to 48MHz */






	  //************************************************************************************
	  //*				Generic clock PLL1
	  pm_pll_setup(pm,
	  	               1,   // use PLL0
	  	              7,   // MUL=7 in the formula
	  	               1,   // DIV=1 in the formula
	  	               0,   // Sel Osc0/PLL0 or Osc1/PLL1
	  	               16); // lockcount in main clock for the PLL wait lock

	  pm_pll_set_option(pm,
						  1, //PLL
						  1, //PLL frequenz
						  4, //PLL DIV
						  0);//WBW

	  pm_pll_enable(pm,1);

	  pm_wait_for_pll1_locked(pm) ;

	  pm_gc_setup(pm,
	                EXAMPLE_GCLK_ID,
	                0,  // Use Osc (=0) or PLL (=1), here PLL
	                0,  // Sel Osc0/PLL0 or Osc1/PLL1
	                0,  // disable divisor
	                0); // no divisor

	  pm_gc_enable(pm, EXAMPLE_GCLK_ID);


	  gpio_enable_module_pin(EXAMPLE_GCLK_PIN, EXAMPLE_GCLK_FUNCTION);
}

/*
 * pm_gc_setup(pm,
              EXAMPLE_GCLK_ID,
              1,  // Use Osc (=0) or PLL (=1), here PLL
              0,  // Sel Osc0/PLL0 or Osc1/PLL1
              0,  // disable divisor
              0); // no divisor


  pm_gc_enable(pm, EXAMPLE_GCLK_ID);


  gpio_enable_module_pin(EXAMPLE_GCLK_PIN, EXAMPLE_GCLK_FUNCTION);
 */
